class Attitude:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_boot_ms = None
            self.roll = None
            self.pitch = None
            self.yaw = None
            self.rollspeed = None
            self.pitchspeed = None
            self.yawspeed = None
        else:
            self.all = message
            self.time_boot_ms = message.time_boot_ms
            self.roll = message.roll
            self.pitch = message.pitch
            self.yaw = message.yaw
            self.rollspeed = message.rollspeed
            self.pitchspeed = message.pitchspeed
            self.yawspeed = message.yawspeed



import time

from pymavlink import mavutil

mode = ""


class DoReposition:
    def __init__(self, connection_object, message_object):
        self.connection_object = connection_object
        self.message = message_object

    def go_waypoint(self, lat, lon, alt):
        self.message.command_ack = None
        self.connection_object.mav.command_int_send(
            0,
            0,
            mavutil.mavlink.MAV_FRAME_GLOBAL_RELATIVE_ALT_INT,
            mavutil.mavlink.MAV_CMD_DO_REPOSITION,
            0,
            0,
            -1,
            0,
            0,
            0,
            int(lat * 1e7),
            int(lon * 1e7),
            alt,
        )
        while self.message.command_ack is None:
            time.sleep(0.1)
        print(self.message.command_ack)



class EkfStatusReport:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.flags = None
            self.velocity_variance = None
            self.pos_horiz_variance = None
            self.pos_vert_variance = None
            self.compass_variance = None
            self.terrain_alt_variance = None
            self.airspeed_variance = None
        else:
            self.all = message
            self.flags = message.flags
            self.velocity_variance = message.velocity_variance
            self.pos_horiz_variance = message.pos_horiz_variance
            self.pos_vert_variance = message.pos_vert_variance
            self.compass_variance = message.compass_variance
            self.terrain_alt_variance = message.terrain_alt_variance
            self.airspeed_variance = message.airspeed_variance



class Heartbeat:
    def __init__(self, gps_message):
        self.message = gps_message
        self.type = self.message.type
        self.autopilot = self.message.autopilot
        self.base_mode = self.message.base_mode
        self.custom_mode = self.message.custom_mode
        self.system_status = self.message.system_status
        self.mavlink_version = self.message.mavlink_version
        self.arm = None

    def get_all_state(self):
        return self.message

    def get_custom_mode(self):
        return self.custom_mode

    def get_autopilot(self):
        return self.autopilot

    def get_base_mode(self):
        return self.base_mode

    def get_system_status(self):
        return self.system_status

    def get_mavlink_version(self):
        return self.mavlink_version
    """
    def is_arm(self):
        En:
            This function controls the arm status
            if system_status is equals 3 it means disarm,
            if system_status is equals 4 it means arm

            :return False if disarm
            :return True if arm
            :return -1(int) if another thing

        Tr:
            Bu fonksiyon arm drumunu kontrol eder
            system_status 3'e eşitse disarm,
            system_status 4'e  eşitse arm
            durumundadır.
        if self.system_status == 4:
            self.arm = True
            return self.arm
        if self.system_status == 3:
            self.arm = False
            return self.arm
        return -1
    """



import time

import planekit
from planekit.mavlink.SendMessage import SendMessage


class SetArm:
    def __init__(self, connection_object, message_object):
        self.connection_object = connection_object
        self.arm_status = None
        self.message = message_object

    def arm(self):
        while self.connection_object.motors_armed() != 128:
            self.arm_status = planekit.ReceiveData(self.connection_object).arm_status()
            send_message = SendMessage(self.connection_object, self.message)
            send_message.run_cmd(
                send_message.mavutil.mavlink.MAV_CMD_COMPONENT_ARM_DISARM,  # command
                1,  # param1 (1 to indicate arm)
                1,  # param2  (all other params meaningless)
                0,  # param3
                0,  # param4
                0,  # param5
                0,  # param6
                0  # param7
            )
            time.sleep(1)

    def disarm(self):
        while self.connection_object.motors_armed() != 0:
            send_message = SendMessage(self.connection_object, self.message)
            send_message.run_cmd(
                send_message.mavutil.mavlink.MAV_CMD_COMPONENT_ARM_DISARM,  # command
                0,  # param1 (1 to indicate arm)
                1,  # param2  (all other params meaningless)
                0,  # param3
                0,  # param4
                0,  # param5
                0,  # param6
                0  # param7
            )



class TerrainReport:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.lat = None
            self.lon = None
            self.spacing = None
            self.terrain_height = None
            self.current_height = None
            self.pending = None
            self.loaded = None
        else:
            self.all = message
            self.lat = message.lat
            self.lon = message.lon
            self.spacing = message.spacing
            self.terrain_height = message.terrain_height
            self.current_height = message.current_height
            self.pending = message.pending
            self.loaded = message.loaded



class HwStatus:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.Vcc = None
            self.I2Cerr = None
        else:
            self.all = message
            self.Vcc = message.Vcc
            self.I2Cerr = message.I2Cerr


import sys

from pymavlink import mavutil
from planekit.exception.ConnectionIsNoneError import ConnectionIsNoneError


class Connection:
    def __init__(self):
        self.connection_object = None
        self.connection_string = None

    def get_connection(self):
        # Remove print
        # print(self.connection_string)
        a = ConnectionIsNoneError(self.connection_string)
        a.is_connection_none()
        # print(self.connection)
        if self.connection_object is None:
            sys.exit()
        return self.connection_object

    def connection(self, connection_string):
        self.connection_string = connection_string
        master = mavutil.mavlink_connection(connection_string)
        master.wait_heartbeat()
        self.connection_object = master
        return self.get_connection()



class BatteryStatus:
    def __init__(self, message=None):
        if message is None:
            self.all = None
            self.temperature = None
            self.voltages = None
            self.current_battery = None
            self.current_consumed = None
            self.energy_consumed = None
            self.battery_remaining = None
            self.time_remaining = None
            self.charge_state = None
            self.voltages_ext = None
            self.mode = None
            self.fault_bitmask = None
        else:
            self.all = message
            self.temperature = message.temperature
            self.voltages = message.voltages
            self.current_battery = message.current_battery
            self.current_consumed = message.current_consumed
            self.energy_consumed = message.energy_consumed
            self.battery_remaining = message.battery_remaining
            self.time_remaining = message.time_remaining
            self.charge_state = message.charge_state
            self.voltages_ext = message.voltages_ext
            self.mode = message.mode
            self.fault_bitmask = message.fault_bitmask



class PowerStatus:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.Vcc = None
            self.Vservo = None
            self.flags = None
        else:
            self.all = message
            self.Vcc = message.Vcc
            self.Vservo = message.Vservo
            self.flags = message.flags



class EscTelemetry:
    def __init__(self, message=None):
        if message is None:
            self.all = None
            self.temperature = None
            self.voltage = None
            self.current = None
            self.totalcurrent = None
        else:
            self.all = message
            self.temperature = message.temperature
            self.voltage = message.voltage
            self.current = message.current
            self.totalcurrent = message.totalcurrent



from threading import Thread

from planekit.data.Ahrs import Ahrs
from planekit.data.Gps import Gps
from planekit.data.Heartbeat import Heartbeat
from planekit.data.Imu import Imu
from planekit.data.Attitude import Attitude
from planekit.data.GlobalPositionInt import GlobalPositionInt
from planekit.data.EscTelemetry import EscTelemetry
from planekit.data.AoaSsa import AoaSsa
from planekit.data.BatteryStatus import BatteryStatus
from planekit.data.PositionTargetGlobalInt import PositionTargetGlobalInt
from planekit.data.Vibration import Vibration
from planekit.data.LocalPositionNed import LocalPositionNed
from planekit.data.EkfStatusReport import EkfStatusReport
from planekit.data.TerrainReport import TerrainReport
from planekit.data.Wind import Wind
from planekit.data.HwStatus import HwStatus
from planekit.data.Ahrs2 import Ahrs2
from planekit.data.Simstate import Simstate
from planekit.data.Ahrs import Ahrs
from planekit.data.GpsRawInt import GpsRawInt
from planekit.data.ScaledPressure2 import ScaledPressure2
from planekit.data.ScaledPressure import ScaledPressure
from planekit.data.ScaledImu3 import ScaledImu3
from planekit.data.ScaledImu2 import ScaledImu2
from planekit.data.RawImu import RawImu
from planekit.data.RcChannels import RcChannels
from planekit.data.ServoOutputRaw import ServoOutputRaw
from planekit.data.MissionCurrent import MissionCurrent
from planekit.data.VfrHud import VfrHud
from planekit.data.PowerMeminfo import PowerMeminfo
from planekit.data.PowerStatus import PowerStatus


# noinspection PyAttributeOutsideInit
class ReceiveData:
    """
    En:
        This class parses mavtcp messages by type and sends them to the corresponding classes.
        :arg connection_object
            Gets the object instance pymavlink.mavutil.mavtcp
    Tr:
        Bu sınıf, mavtcp mesajlarını get_type ile ayrıştırır ve bunları ilgili sınıflara gönderir.
        :arg connection_object
            pymavlink.mavutil.mavtcp den türetilmiş objeyi alır
    """

    def __init__(self, connection_object):
        self.connection_object = connection_object
        self.message_object = None

        self.attitude = Attitude()
        self.global_position = GlobalPositionInt()
        self.esc_telemetry = EscTelemetry()
        self.aoa_ssa = AoaSsa()
        self.battery_status = BatteryStatus()
        self.position_target_global_int = PositionTargetGlobalInt()
        self.vibration = Vibration()
        self.local_position_end = LocalPositionNed()
        self.ekf_status_report = EkfStatusReport()
        self.terrain_report = TerrainReport()
        self.wind = Wind()
        self.hwstatus = HwStatus()
        self.ahrs2 = Ahrs2()
        self.simstate = Simstate()
        self.ahrs = Ahrs()
        # Aşağıdakiler için örnekler yazılacak
        self.gps_raw_int = GpsRawInt()
        self.scaled_pressure2 = ScaledPressure2()
        self.scaled_pressure = ScaledPressure()
        self.scaled_imu3 = ScaledImu3()
        self.scaled_imu2 = ScaledImu2()
        self.raw_imu = RawImu()
        self.rc_channels = RcChannels()
        self.servo_output_raw = ServoOutputRaw()
        self.mission_current = MissionCurrent()
        self.vfr_hud = VfrHud()
        self.power_meminfo = PowerMeminfo()
        self.power_status = PowerStatus()
        self.heartbeat_message = Heartbeat()
        self.command_ack = None
        self.mission_request = None
        '''
        ! None dönenler
        self.mav_controller_output = None
        self.param_value = None
        self.airspeed_autocal = None
        self.system_time = None
        '''
        self.imu_message = None
        self.gps_message = None
        # self.heartbeat_message = None
        self.ahrs_message = None

    def get_message(self):
        while True:
            self.message_object = self.connection_object.recv_match()
            if self.message_object is not None and self.message_object.get_type() == 'RAW_IMU':
                self.raw_imu = RawImu(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'COMMAND_ACK':
                self.command_ack = self.message_object

            if self.message_object is not None and self.message_object.get_type() == 'MISSION_REQUEST':
                self.mission_request = self.message_object

            if self.message_object is not None and self.message_object.get_type() == 'ATTITUDE':
                self.attitude = Attitude(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'GLOBAL_POSITION_INT':
                self.global_position = GlobalPositionInt(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SYS_STATUS':
                self.sys_status = self.message_object

            if self.message_object is not None and self.message_object.get_type() == 'POWER_STATUS':
                self.power_status = PowerStatus(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'MEMINFO':
                self.power_meminfo = PowerMeminfo(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'NAV_CONTROLLER_OUTPUT':
                self.mav_controller_output = self.message_object

            if self.message_object is not None and self.message_object.get_type() == 'MISSION_CURRENT':
                self.mission_current = MissionCurrent(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'VFR_HUD':
                self.vfr_hud = VfrHud(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SERVO_OUTPUT_RAW':
                self.servo_output_raw = ServoOutputRaw(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'RC_CHANNELS':
                self.rc_channels = RcChannels(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SCALED_IMU2':
                self.scaled_imu2 = ScaledImu2(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SCALED_IMU3':
                self.scaled_imu3 = ScaledImu3(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SCALED_PRESSURE':
                self.scaled_pressure = ScaledPressure(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SCALED_PRESSURE2':
                self.scaled_pressure2 = ScaledPressure2(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'GPS_RAW_INT':
                self.gps_raw_int = GpsRawInt(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SYSTEM_TIME':
                self.system_time = self.message_object

            if self.message_object is not None and self.message_object.get_type() == 'AHRS':
                self.ahrs = Ahrs(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'SIMSTATE':
                self.simstate = Simstate(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'AHRS2':
                self.ahrs2 = Ahrs2(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'HWSTATUS':
                self.hwstatus = HwStatus(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'WIND':
                self.wind = Wind(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'TERRAIN_REPORT':
                self.terrain_report = TerrainReport(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'EKF_STATUS_REPORT':
                self.ekf_status_report = EkfStatusReport(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'LOCAL_POSITION_NED':
                self.local_position_end = LocalPositionNed(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'VIBRATION':
                self.vibration = Vibration(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'POSITION_TARGET_GLOBAL_INT':
                self.position_target_global_int = PositionTargetGlobalInt(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'BATTERY_STATUS':
                self.battery_status = BatteryStatus(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'AOA_SSA':
                self.aoa_ssa = AoaSsa(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'ESC_TELEMETRY_1_TO_4':
                self.esc_telemetry = EscTelemetry(self.message_object)

            if self.message_object is not None and self.message_object.get_type() == 'AIRSPEED_AUTOCAL':
                self.airspeed_autocal = self.message_object

            if self.message_object is not None and self.message_object.get_type() == 'PARAM_VALUE':
                self.param_value = self.message_object

            if self.message_object is not None and self.message_object.get_type() == 'HEARTBEAT':
                self.heartbeat_message = Heartbeat(self.message_object)

    def start_data_thread(self):
        t = Thread(target=self.get_message)
        t.start()
        t = Thread()
        t.start()

    """
    Oldest
    """

    def get_message_object(self):
        """
        en:
            :return:
            self.message_object : Return messages received with recv_match.
        tr:
            :return:
                self.message_object : recv_match ile alınan mesajları dönderir.
        """

        self.message_object = self.connection_object.recv_match()
        return self.message_object

    def select_imu_message(self):
        """
        En:
            This func. selects imu messages received from self.get_message_object.
        Tr:
            Bu func. self.get_message_object'den alınan imu mesajlarını seçer.
        """

        return self.raw_imu

    def select_gps_message(self):
        """
            En:
                This func. selects gps messages received from self.get_message_object.
            Tr:
                Bu func. self.get_message_object'den alınan gps mesajlarını seçer.
        """
        return self.gps_raw_int

    def select_heartbeat_message(self):
        print("error")
        return self.heartbeat_message

    def select_ahrs_message(self):
        return self.ahrs2

    """
    def arm_status(self):
            En:
                Heartbeat(self.heartbeat_message).arm can have 3 values.
                These are True(if arm) False(if disarm) and -1(another thing).
            Tr:
                Heartbeat(self.heartbeat_message).arm 3 değere sahip olabilir.
                Bunlar True(if arm) False(if disarm) ve -1(another thing) olabilir.

        while True:
            self.heartbeat_message = planekit.ReceiveData(self.connection_object).get_message_object()
            if self.heartbeat_message is not None and self.heartbeat_message.get_type() == 'HEARTBEAT':
                heartbeat_message = Heartbeat(self.heartbeat_message)
                if heartbeat_message.is_arm() != -1:
                    return heartbeat_message.is_arm()
    """

    def arm_status(self):
        if self.connection_object.motors_armed() == 128:
            print("disarmed")
            return True
        if self.connection_object.motors_armed() == 0:
            print("armed")
            return False

    def ground_speed_message(self):
        """
            En:
                :return:
                    Return the vel parameter from messages with type GPS_RAW_INT
            Tr:
                :return:
                    GPS_RAW_INT tipine sahip mesajlardan vel parametresini dönderir
        """

        return self.gps_raw_int.vel

    def alt(self, relative=True, timeout=30):
        return self.global_position.relative_alt / 1000



class Ahrs:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.omegaIx = None
            self.omegaIy = None
            self.omegaIz = None
            self.accel_weight = None
            self.renorm_val = None
            self.error_rp = None
            self.error_yaw = None
        else:
            self.all = message
            self.omegaIx = message.omegaIx
            self.omegaIy = message.omegaIy
            self.omegaIz = message.omegaIz
            self.accel_weight = message.accel_weight
            self.renorm_val = message.renorm_val
            self.error_rp = message.error_rp
            self.error_yaw = message.error_yaw



class ScaledImu2:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_boot_ms = None
            self.xacc = None
            self.yacc = None
            self.zacc = None
            self.xgyro = None
            self.ygyro = None
            self.zgyro = None
            self.xmag = None
            self.ymag = None
            self.zmag = None
            self.temperature = None
        else:
            self.all = message
            self.time_boot_ms = message.time_boot_ms
            self.xacc = message.xacc
            self.yacc = message.yacc
            self.zacc = message.zacc
            self.xgyro = message.xgyro
            self.ygyro = message.ygyro
            self.zgyro = message.zgyro
            self.xmag = message.xmag
            self.ymag = message.ymag
            self.zmag = message.zmag
            self.temperature = message.temperature



class MissionCurrent:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.seq = None
        else:
            self.all = message
            self.seq = message.seq



class RcChannels:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_boot_ms = None
            self.chancount = None
            self.chan1_raw = None
            self.chan2_raw = None
            self.chan3_raw = None
            self.chan4_raw = None
            self.chan5_raw = None
            self.chan6_raw = None
            self.chan7_raw = None
            self.chan8_raw = None
            self.chan9_raw = None
            self.chan10_raw = None
            self.chan11_raw = None
            self.chan12_raw = None
            self.chan13_raw = None
            self.chan14_raw = None
            self.chan15_raw = None
            self.chan16_raw = None
            self.chan17_raw = None
            self.chan18_raw = None
            self.rssi = None
        else:
            self.all = message
            self.time_boot_ms = message.time_boot_ms
            self.chancount = message.chancount
            self.chan1_raw = message.chan1_raw
            self.chan2_raw = message.chan2_raw
            self.chan3_raw = message.chan3_raw
            self.chan4_raw = message.chan4_raw
            self.chan5_raw = message.chan5_raw
            self.chan6_raw = message.chan6_raw
            self.chan7_raw = message.chan7_raw
            self.chan8_raw = message.chan8_raw
            self.chan9_raw = message.chan9_raw
            self.chan10_raw = message.chan10_raw
            self.chan11_raw = message.chan11_raw
            self.chan12_raw = message.chan12_raw
            self.chan13_raw = message.chan13_raw
            self.chan14_raw = message.chan14_raw
            self.chan15_raw = message.chan15_raw
            self.chan16_raw = message.chan16_raw
            self.chan17_raw = message.chan17_raw
            self.chan18_raw = message.chan18_raw
            self.rssi = message.rssi



class GlobalPositionInt:
    def __init__(self, message=None):
        if message is None:
            self.all = None
            self.time_boot_ms = None
            self.lat = None
            self.lon = None
            self.alt = None
            self.relative_alt = None
            self.vx = None
            self.vy = None
            self.vz = None
            self.hdg = None
        else:
            self.all = message
            self.time_boot_ms = message.time_boot_ms
            self.lat = message.lat
            self.lon = message.lon
            self.alt = message.alt
            self.relative_alt = message.relative_alt
            self.vx = message.vx
            self.vy = message.vy
            self.vz = message.vz
            self.hdg = message.hdg



class LocalPositionNed:
    def __init__(self, message=None):
        if message is None:
            self.all = None
            self.time_boot_ms = None
            self.x = None
            self.y = None
            self.z = None
            self.vx = None
            self.vy = None
            self.vz = None

        else:
            self.all = message
            self.time_boot_ms = message.time_boot_ms
            self.x = message.z
            self.y = message.y
            self.z = message.z
            self.vx = message.vx
            self.vy = message.vy
            self.vz = message.vz



class AoaSsa:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_usec = None
            self.AOA = None
            self.SSA = None
        else:
            self.all = message
            self.time_usec = message.time_usec
            self.AOA = message.AOA
            self.SSA = message.SSA



class Vibration:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_usec = None
            self.vibration_x = None
            self.vibration_y = None
            self.vibration_z = None
            self.clipping_0 = None
            self.clipping_1 = None
            self.clipping_2 = None
        else:
            self.all = message
            self.time_usec = message.time_usec
            self.vibration_x = message.vibration_x
            self.vibration_y = message.vibration_y
            self.vibration_z = message.vibration_z
            self.clipping_0 = message.clipping_0
            self.clipping_1 = message.clipping_1
            self.clipping_2 = message.clipping_2



class GpsRawInt:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_usec = None
            self.fix_type = None
            self.lat = None
            self.lon = None
            self.alt = None
            self.eph = None
            self.epv = None
            self.vel = None
            self.cog = None
            self.satellites_visible = None
            self.alt_ellipsoid = None
            self.h_acc = None
            self.v_acc = None
            self.vel_acc = None
            self.hdg_acc = None
            self.yaw = None
        else:
            self.all = message
            self.time_usec = message.time_usec
            self.fix_type = message.fix_type
            self.lat = message.lat
            self.lon = message.lon
            self.alt = message.alt
            self.eph = message.eph
            self.epv = message.epv
            self.vel = message.vel
            self.cog = message.cog
            self.satellites_visible = message.satellites_visible
            self.alt_ellipsoid = message.alt_ellipsoid
            self.h_acc = message.h_acc
            self.v_acc = message.v_acc
            self.vel_acc = message.vel_acc
            self.hdg_acc = message.hdg_acc
            self.yaw = message.yaw



import sys
import time

from planekit.mavlink.SendMessage import SendMessage
from planekit.mavlink.ReceiveData import ReceiveData

mode = ""


class FlightMode:
    def __init__(self, connection_object, message_object):
        self.connection_object = connection_object
        self.arm_status = None
        self.message = message_object

    def set_flight_mode(self, mode):
        # Choose a mode
        send_message = SendMessage(self.connection_object, self.message)
        if mode not in self.connection_object.mode_mapping():
            print('Unknown mode : {}'.format(mode))
            print('Try:', list(self.connection_object.mode_mapping().keys()))
            sys.exit(1)

        self.message.command_ack = None
        mode_id = self.connection_object.mode_mapping()[mode]
        # Set new mode
        # self.connection_object.mav.command_long_send(
        #    self.connection_object.target_system, self.connection_object.target_component,
        #    mavutil.mavlink.MAV_CMD_DO_SET_MODE, 0,
        #    0, mode_id, 0, 0, 0, 0, 0) or:
        # self.connection_object.set_mode(mode_id) or:
        self.connection_object.mav.set_mode_send(
            self.connection_object.target_system,
            send_message.mavutil.mavlink.MAV_MODE_FLAG_CUSTOM_MODE_ENABLED,
            mode_id)
        # Wait for ACK command
        # Would be good to add mechanism to avoid endlessly blocking
        # if the autopilot sends a NACK or never receives the message
        while self.message.command_ack is None:
            time.sleep(0.1)
        print(self.message.command_ack)

    def get_flight_mode(self):
        global mode
        all_mode = ['MANUAL', 'CIRCLE', 'STABILIZE', 'TRAINING', 'ACRO', 'FBWA', 'FBWB', 'CRUISE', 'AUTOTUNE', "AUTO",
                    'AUTO',
                    'RTL', 'LOITER', 'TAKEOFF', 'AVOID_ADSB', 'GUIDED', 'INITIALISING', 'QSTABILIZE', 'QHOVER',
                    'QLOITER', 'QLAND', 'QRTL', 'QAUTOTUNE', 'QACRO', 'THERMAL', 'LOITERALTQLAND']
        if self.message.heartbeat_message.all.type == 1:
            mode = all_mode[self.message.heartbeat_message.all.custom_mode]
        return mode



import time
from planekit.Calculator import Calculator
from planekit.command.DoReposition import DoReposition
from planekit.command.FightMode import FlightMode
from planekit.command.SetArm import SetArm
from planekit.command.TakeOffLandCMD import TakeOffLandCMD
from planekit.data.Imu import Imu
from planekit.mavlink.Connection import Connection as MavLinkConnection
from planekit.mavlink.ReceiveData import ReceiveData
from planekit.mavlink.SendMessage import SendMessage


class Connection:
    def __init__(self, connection_string):
        self.connection_string = connection_string
        self.mavlink = MavLinkConnection()
        self.connection_object = self.mavlink.connection(connection_string)
        self.message = ReceiveData(self.connection_object)
        self.message.start_data_thread()

    def arm(self):
        set_arm = SetArm(self.connection_object, self.message)
        set_arm.arm()

    def disarm(self):
        set_arm = SetArm(self.connection_object, self.message)
        set_arm.disarm()

    """
    def takeoff(self, alt):
        takeoff = TakeOffLandCMD(self.connection_object, self.message)
        takeoff.takeoff_string(alt)
        self.set_flight_mode("AUTO")
        while self.mode() != "AUTO":
            print("mode ayarlanıyor")
            self.set_flight_mode("AUTO")
            time.sleep(1)
    """

    def set_flight_mode(self, mode):
        set_flight_modes = FlightMode(self.connection_object, self.message)
        set_flight_modes.set_flight_mode(mode)

    def mode(self):
        return FlightMode(self.connection_object, self.message).get_flight_mode()

    def set_rc_channel_pwm(self, channel_id, pwm=1500):
        """ Set RC channel pwm value
        Args:
            channel_id (TYPE): Channel ID
            pwm (int, optional): Channel pwm value 1100-1900
        """
        if channel_id < 1 or channel_id > 18:
            print("Channel does not exist.")
            return

        rc_channel_values = [65535 for _ in range(18)]
        rc_channel_values[channel_id - 1] = pwm
        self.connection_object.mav.rc_channels_override_send(
            self.connection_object.target_system,  # target_system
            self.connection_object.target_component,  # target_component
            *rc_channel_values)  # RC channel list, in microseconds.


"""


class Connection:
    def __init__(self, connection_string):
        self.connection_string = connection_string

        self.mavlink = MavLinkConnection()
        self.connection_object = self.mavlink.connection(connection_string)

        self.message = ReceiveData(self.connection_object)
        self.message.start_data_thread()

    def wait_heartbeat(self):
        m = SendMessage(self.connection_object, self.message)
        m.wait_heartbeat()

    # Test remove prod.
    def get_connection(self):
        self.mavlink.get_connection()

    def messages(self):
        return self.message

    def imu_message(self):
        return ReceiveData(self.connection_object).select_imu_message()

    def gps_message(self):
        return ReceiveData(self.connection_object).select_gps_message()

    def heartbeat_message(self):
        return ReceiveData(self.connection_object).select_heartbeat_message()

    def ahrs_message(self):
        return ReceiveData(self.connection_object).select_ahrs_message()

    def lat(self):
        return ReceiveData(self.connection_object).select_gps_message().lat / 10000000

    def lon(self):
        return ReceiveData(self.connection_object).select_gps_message().lon / 10000000

    def alt(self):
        return ReceiveData(self.connection_object).alt()

    def ground_speed(self):
        return ReceiveData(self.connection_object).ground_speed_message() / 100

    def arm_status(self):
        return ReceiveData(self.connection_object).arm_status()

    def mode(self):
        return FlightMode(self.connection_object,self.message).get_flight_mode()

    def arm(self):
        set_arm = SetArm(self.connection_object, self.message)
        set_arm.arm()

    def disarm(self):
        set_arm = SetArm(self.connection_object, self.message)
        set_arm.disarm()

    def set_flight_mode(self, mode):
        set_flight_modes = FlightMode(self.connection_object, self.message)
        set_flight_modes.set_flight_mode(mode)

    def takeoff_and_arm(self, alt):
        takeoff = TakeOffLandCMD(self.connection_object, self.message)
        takeoff.takeoff_string(alt)
        self.arm()
        self.set_flight_mode("AUTO")
        while self.mode() != "AUTO":
            # print(self.mode())
            self.set_flight_mode("AUTO")
            time.sleep(10)

    def land_and_disarm(self):
        while self.arm_status() is True:
            land = TakeOffLandCMD(self.connection_object)
            land.lend_string(12)
            # print(self.mode())
            print("landing")
            self.set_flight_mode("AUTO")
            time.sleep(10)

    def go_to(self, lat, lon, alt, sleep=False):
        reposition = DoReposition(self.connection_object,self.message)
        reposition.go_waypoint(lat, lon, alt)
        while sleep:
            a = self.gps_message()
            print(Calculator.haversine(lat, lon, a.lat / 10000000, a.lon / 10000000))
            if Calculator.haversine(lat, lon, a.lat / 10000000, a.lon / 10000000) < 0.200:
                break

    def a(self):
        return self.connection_object

"""



class PositionTargetGlobalInt:
    def __init__(self, message=None):
        if message is None:
            self.all = None
            self.time_boot_ms = None
            self.coordinate_frame = None
            self.type_mask = None
            self.lon_int = None
            self.alt = None
            self.vx = None
            self.vy = None
            self.vz = None
            self.afx = None
            self.afy = None
            self.afz = None
            self.yaw = None
            self.yaw_rate = None
        else:
            self.all = message
            self.time_boot_ms = message.time_boot_ms
            self.coordinate_frame = message.coordinate_frame
            self.type_mask = message.type_mask
            self.lon_int = message.lon_int
            self.alt = message.alt
            self.vx = message.vx
            self.vy = message.vy
            self.vz = message.vz
            self.afx = message.afx
            self.afy = message.afy
            self.afz = message.afz
            self.yaw = message.yaw
            self.yaw_rate = message.yaw_rate



import time

from pymavlink import mavutil
from pymavlink import mavwp


class TakeOffLandCMD:
    def __init__(self, connection_object, message_object):
        self.connection_object = connection_object
        self.message = message_object
        self.wp = mavwp.MAVWPLoader()

    def cmd_set_home(self, home_location, altitude):
        # print('--- ', self.connection_object.target_system, ',', self.connection_object.target_component)
        self.connection_object.mav.command_long_send(
            self.connection_object.target_system, self.connection_object.target_component,
            mavutil.mavlink.MAV_CMD_DO_SET_HOME,
            1,  # set position
            0,  # param1
            0,  # param2
            0,  # param3
            0,  # param4
            home_location[0],  # lat
            home_location[1],  # lon
            altitude)

    def upload_mission(self, lines):
        home_location = None
        home_altitude = None
        # print(lines)
        for i in range(0, 2):

            # print("*************************************")
            line = lines[i]
            mission_param = line.split('\t')
            # print(mission_param)
            ln_seq = int(mission_param[0])
            ln_current = int(mission_param[1])
            ln_frame = int(mission_param[2])
            ln_command = int(mission_param[3])
            ln_param1 = float(mission_param[4])
            ln_param2 = float(mission_param[5])
            ln_param3 = float(mission_param[6])
            ln_param4 = float(mission_param[7])
            ln_x = float(mission_param[8])
            ln_y = float(mission_param[9])
            ln_z = float(mission_param[10])
            ln_autocontinue = int(float(mission_param[11].strip()))
            if i == 1:
                home_location = (ln_x, ln_y)
                home_altitude = ln_z
            self.message.command_ack = None
            # print("msg"+str(self.message.command_ack))
            p = mavutil.mavlink.MAVLink_mission_item_message(self.connection_object.target_system,
                                                             self.connection_object.target_component, ln_seq,
                                                             ln_frame,
                                                             ln_command,
                                                             ln_current, ln_autocontinue, ln_param1, ln_param2,
                                                             ln_param3, ln_param4, ln_x, ln_y, ln_z)
            self.wp.add(p)

        self.cmd_set_home(home_location, home_altitude)
        print("1")
        while self.message.command_ack is  None:
            time.sleep(.1)
        str(self.message.command_ack)
        # print('Set home location: {0} {1}'.format(home_location[0], home_location[1]))
        time.sleep(1)
        print("al")
        self.message.mission_request = None
        # send waypoint to airframe
        self.connection_object.waypoint_clear_all_send()
        self.connection_object.waypoint_count_send(self.wp.count())

        for i in range(self.wp.count()):
            while self.message.mission_request is None:
                time.sleep(.1)
            str(self.message.mission_request)
            self.message.mission_request = None

    def takeoff_string(self, a):
        print("ssssssss")
        line = ["0	1	0	16	0	0	0	0	35.3633498	-149.1652373	585.040000	1",
                "1	0	3	22	40.00000000	0.00000000	0.00000000	0.00000000	-35.36574580	149.15974620	" + str(
                    a) + "	1"]

        # print(line)
        self.upload_mission(line)

    def lend_string(self, a):
        line = ["0	1	0	16	0	0	0	0	35.3633497	-149.1652372	585.039978	1",
                "1	0	3	21	0.00000000	0.00000000	0.00000000	1.00000000	-35.36329600	149.15901660	0.000000	1"]

        # print(line)
        self.upload_mission(line)



from math import radians, cos, sin, asin, sqrt


class Calculator:

    @staticmethod
    def haversine(lat1, lon1, lat2, lon2):
        """
        Calculate the great circle distance in kilometers between two points
        on the earth (specified in decimal degrees)
        """
        # convert decimal degrees to radians
        lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])

        # haversine formula
        dlon = lon2 - lon1
        dlat = lat2 - lat1
        a = sin(dlat / 2) ** 2 + cos(lat1) * cos(lat2) * sin(dlon / 2) ** 2
        c = 2 * asin(sqrt(a))
        r = 6371  # Radius of earth in kilometers. Use 3956 for miles. Determines return value units.
        return c * r



class Wind:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.direction = None
            self.speed = None
            self.speed_z = None
        else:
            self.all = message
            self.direction = message.direction
            self.speed = message.speed
            self.speed_z = message.speed_z



class ScaledPressure:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_boot_ms = None
            self.press_abs = None
            self.press_diff = None
            self.temperature = None
            self.temperature_press_diff = None
        else:
            self.all = message
            self.time_boot_ms = message.time_boot_ms
            self.press_abs = message.press_abs
            self.press_diff = message.press_diff
            self.temperature = message.temperature
            self.temperature_press_diff = message.temperature_press_diff



class Simstate:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.roll = None
            self.pitch = None
            self.yaw = None
            self.xacc = None
            self.yacc = None
            self.zacc = None
            self.xgyro = None
            self.ygyro = None
            self.zgyro = None
            self.lat = None
            self.lng = None
        else:
            self.all = message
            self.roll = message.roll
            self.pitch = message.pitch
            self.yaw = message.yaw
            self.xacc = message.xacc
            self.yacc = message.yacc
            self.zacc = message.zacc
            self.xgyro = message.xgyro
            self.ygyro = message.ygyro
            self.zgyro = message.zgyro
            self.lat = message.lat
            self.lng = message.lng



class PowerMeminfo:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.brkval = None
            self.freemem = None
            self.freemem32 = None
        else:
            self.all = message
            self.brkval = message.brkval
            self.freemem = message.freemem
            self.freemem32 = message.freemem32



class VfrHud:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.airspeed = None
            self.groundspeed = None
            self.heading = None
            self.throttle = None
            self.alt = None
            self.climb = None
        else:
            self.all = message
            self.airspeed = message.airspeed
            self.groundspeed = message.groundspeed
            self.heading = message.heading
            self.throttle = message.throttle
            self.alt = message.alt
            self.climb = message.climb



class ServoOutputRaw:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.time_usec = None
            self.port = None
            self.servo1_raw = None
            self.servo2_raw = None
            self.servo3_raw = None
            self.servo4_raw = None
            self.servo5_raw = None
            self.servo6_raw = None
            self.servo7_raw = None
            self.servo8_raw = None
            self.servo9_raw = None
            self.servo10_raw = None
            self.servo11_raw = None
            self.servo12_raw = None
            self.servo13_raw = None
            self.servo14_raw = None
            self.servo15_raw = None
            self.servo16_raw = None
        else:
            self.all = message
            self.time_usec = message.time_usec
            self.port = message.port
            self.servo1_raw = message.servo1_raw
            self.servo2_raw = message.servo2_raw
            self.servo3_raw = message.servo3_raw
            self.servo4_raw = message.servo4_raw
            self.servo5_raw = message.servo5_raw
            self.servo6_raw = message.servo6_raw
            self.servo7_raw = message.servo7_raw
            self.servo8_raw = message.servo8_raw
            self.servo9_raw = message.servo9_raw
            self.servo10_raw = message.servo10_raw
            self.servo11_raw = message.servo11_raw
            self.servo12_raw = message.servo12_raw
            self.servo13_raw = message.servo13_raw
            self.servo14_raw = message.servo14_raw
            self.servo15_raw = message.servo15_raw
            self.servo16_raw = message.servo16_raw



class Ahrs2:
    def __init__(self, message=None):
        if message is None:
            self.all = message
            self.roll = None
            self.pitch = None
            self.yaw = None
            self.altitude = None
            self.lat = None
            self.lng = None
        else:
            self.all = message
            self.roll = message.roll
            self.pitch = message.pitch
            self.yaw = message.yaw
            self.altitude = message.altitude
            self.lat = message.lat
            self.lng = message.lng
